### R code from vignette source 'baddeley.Rnw'

###################################################
### code chunk number 1: baddeley.Rnw:14-15
###################################################
options(useFancyQuotes=FALSE)


###################################################
### code chunk number 2: baddeley.Rnw:19-26
###################################################
opt.pp <- function() {
  options(SweaveHooks=list(fig=function() par(mar=rep(2, 4))))
}
opt.fun <- function() {
  options(SweaveHooks=list(fig=function() par(mar=c(5,4,2,3)+0.2)))
}
opt.pp()


###################################################
### code chunk number 3: baddeley.Rnw:64-65
###################################################
if(!file.exists("baddeley-figures")) dir.create("baddeley-figures")


###################################################
### code chunk number 4: startup
###################################################
library(spatstat)
if(versionstring.spatstat() < "1.32") {
  stop("spatstat version 1.32-0 or later is required")
}


###################################################
### code chunk number 5: baddeley.Rnw:81-82
###################################################
recompute <- FALSE


###################################################
### code chunk number 6: baddeley.Rnw:89-90
###################################################
set.seed(54321)


###################################################
### code chunk number 7: baddeley.Rnw:97-99
###################################################
Y <- rStrauss(beta=250/(100^2), gamma=0.5, R=4, W=square(100))
unitname(Y) <- c("metre", "metres")


###################################################
### code chunk number 8: baddeley.Rnw:104-105
###################################################
opt.pp()


###################################################
### code chunk number 9: Fig1
###################################################
plot(Y, pch=16, main="")


###################################################
### code chunk number 10: baddeley.Rnw:110-111
###################################################
opt.fun()


###################################################
### code chunk number 11: baddeley.Rnw:119-120
###################################################
ELp <- envelope(Y, Lest, nsim=39, verbose=FALSE)


###################################################
### code chunk number 12: Fig2a
###################################################
plot(ELp, xlim=c(0,10), main="", lwd=2, legend=FALSE)


###################################################
### code chunk number 13: Fig2b
###################################################
plot(ELp, . - r ~ r, xlim=c(0,10), main="", lwd=2, legend=FALSE)


###################################################
### code chunk number 14: baddeley.Rnw:140-141
###################################################
ELg <- envelope(Y, Lest, nsim=19, global=TRUE, ginterval=c(0,10), verbose=FALSE)


###################################################
### code chunk number 15: Fig3a
###################################################
plot(ELg, xlim=c(0,10), main="", lwd=2, legend=FALSE)


###################################################
### code chunk number 16: Fig3b
###################################################
plot(ELg, . - r ~ r, xlim=c(0,10), main="", lwd=2, legend=FALSE)


###################################################
### code chunk number 17: baddeley.Rnw:159-160
###################################################
LCI <- lohboot(Y, "Lest", nsim=2000, confidence=0.95)


###################################################
### code chunk number 18: Fig4a
###################################################
plot(LCI, main="", legend=FALSE, xlim=c(0,10))


###################################################
### code chunk number 19: Fig4b
###################################################
plot(LCI, . - r ~ r, main="", legend=FALSE, xlim=c(0,10))


###################################################
### code chunk number 20: baddeley.Rnw:178-179
###################################################
erep19 <- dclf.progress(Y, Lest, nsim=19, verbose=FALSE)


###################################################
### code chunk number 21: Fig5a
###################################################
plot(erep19, main="", xlim=c(0,25), lwd=2, legend=FALSE)


###################################################
### code chunk number 22: erep1999
###################################################
erep1999 <- dclf.progress(Y, Lest, nsim=1999, nrank=100, verbose=FALSE)


###################################################
### code chunk number 23: Fig5b
###################################################
plot(erep1999, main="", xlim=c(0,25), lwd=2, legend=FALSE)


###################################################
### code chunk number 24: baddeley.Rnw:203-204
###################################################
Ntest <- 1000


###################################################
### code chunk number 25: baddeley.Rnw:214-218
###################################################
Gstab <- function(X, ...) {
  GX <- Gest(X, ...)
  eval.fv(asin(sqrt(GX)))
}


###################################################
### code chunk number 26: baddeley.Rnw:224-247
###################################################
powersim <- function(expr, N, fun=Lest, test=c("dclf", "mad"), nsim=19, 
                     rmax=25, nr=512) {
  # validate arguments
  stopifnot(is.expression(expr))
  test <- match.arg(test)
  progfun <- switch(test, dclf=dclf.progress, mad=mad.progress)
  # create space to store results
  reject <- matrix(FALSE, nr, N)
  # distance values 'r' (this is not normally required but saves time)
  r <- seq(0, rmax, length=nr)
  # run loop
  for(i in 1:N) {
    # generate realisation from alternative
    X <- eval(expr)
    # compute test statistic and critical value for each [0,R]
    pr <- progfun(X, fun, nsim=19, r=r, verbose=FALSE)
    # determine outcome of test for each [0,R]
    reject[,i] <- with(pr, obs >= crit)
  }
  # compute power
  power <- rowMeans(reject, na.rm=TRUE)
  return(list(x=r, y=power))
}


###################################################
### code chunk number 27: rinhib
###################################################
if(recompute || !file.exists("powerinhib.rda")) {
  W <- square(100)
  expr <- expression(rStrauss(beta=0.025, gamma=0.5, R=4, W=W))
  powerIDL <- powersim(expr, Ntest, fun=Lest, test="dclf")
  powerIDG <- powersim(expr, Ntest, fun=Gstab, test="dclf")
  powerIML <- powersim(expr, Ntest, fun=Lest, test="mad")
  powerIMG <- powersim(expr, Ntest, fun=Gstab, test="mad")
  save(powerIDL, powerIDG, powerIML, powerIMG, file="powerinhib.rda")
} else load("powerinhib.rda")


###################################################
### code chunk number 28: Fig6a
###################################################
plot(powerIDL, type="l", 
     main="", xlab="r", ylab="Power", 
     ylim=c(0,1), xlim=c(1,25), 
     lwd=2)
lines(powerIML, lwd=2, lty=3)


###################################################
### code chunk number 29: Fig6b
###################################################
plot(powerIDG, type="l", 
     main="", xlab="r", ylab="Power", 
     ylim=c(0,1), xlim=c(1,25), lwd=2)
lines(powerIMG, lwd=2, lty=3)


###################################################
### code chunk number 30: rcluster
###################################################
if(recompute || !file.exists("powerclust.rda")) {
  W <- square(100)
  expr <- expression(rMatClust(kappa=0.005, r=14, mu=5, win=W))
  powerCDL <- powersim(expr, Ntest, fun=Lest, test="dclf")
  powerCDG <- powersim(expr, Ntest, fun=Gstab, test="dclf")
  powerCML <- powersim(expr, Ntest, fun=Lest, test="mad")
  powerCMG <- powersim(expr, Ntest, fun=Gstab, test="mad")
  save(powerCDL, powerCDG, powerCML, powerCMG, file="powerclust.rda")
} else load("powerclust.rda")


###################################################
### code chunk number 31: Fig7a
###################################################
plot(powerCDL, type="l",
     main="", xlab="r", ylab="Power", 
     ylim=c(0,1), xlim=c(1,25), lwd=2)
lines(powerCML, lwd=2, lty=3)


###################################################
### code chunk number 32: Fig7b
###################################################
plot(powerCDG, type="l",
     main="", xlab="r", ylab="Power", 
     ylim=c(0,1), xlim=c(1,25), lwd=2)
lines(powerCMG, lwd=2, lty=3)


###################################################
### code chunk number 33: baddeley.Rnw:339-356
###################################################
Kmcmc <- function(expr, nsim=100) {
  X <- eval(expr)
  Y <- envelope(X, Kest, nsim=nsim,
                simulate=expr,
                weights=npairs, VARIANCE=TRUE)
  # reformat
  Y <- vanilla.fv(Y)
  fvnames(Y, ".y") <- "mmean"
  fvnames(Y, ".s") <- c("loCI", "hiCI")
  discard <- names(Y) %in% c("obs", "lo", "hi")
  Y <- Y[, !discard]
  return(Y)
}
npairs <- function(X) {
  n <- npoints(X)
  return(n * (n-1))
}


###################################################
### code chunk number 34: baddeley.Rnw:362-367
###################################################
beta <- 0.06
R <- 4
W    <- square(100)
Wbig <- grow.rectangle(W, 2 * R)
simexpr <- expression(rHardcore(beta=beta, R=R, W=Wbig)[W])


###################################################
### code chunk number 35: baddeley.Rnw:378-383
###################################################
Nsim <- 100
if(recompute || !file.exists("Khard.rda")) {
  Khard <- Kmcmc(simexpr, Nsim)
  save(Khard, file="Khard.rda")
} else load("Khard.rda")


###################################################
### code chunk number 36: Fig8a
###################################################
plot(Khard,
     sqrt(cbind(mmean, loCI, hiCI)/pi) -r ~ r,
     ylab=expression(L(r)-r), 
     main="", legend=FALSE, lwd=2)


###################################################
### code chunk number 37: baddeley.Rnw:399-400
###################################################
DKhard <- deriv(Khard, which="mmean", kinks=4, spar=0.4)


###################################################
### code chunk number 38: Fig8b
###################################################
plot(DKhard, 
     cbind(mmean/(2 * pi * r), 1) ~ r, 
     ylab=expression(g(r)), 
     col=1, lty=c(1,3), lwd=2,
     main="", legend=FALSE)


###################################################
### code chunk number 39: baddeley.Rnw:425-502
###################################################
CSR.ranks.dclf <- function(Npatterns=1000, 
                           ...,
                           nmc=19,
                           lambda=0.05,
                           W = square(100),
                           rmaxK=25,
                           rmaxG=10,
                           verbose=TRUE) {
  mKpois <- mGpois <- mKcond <- mGcond <-
    mLpois <- mLcond <- mHpois <- mHcond <- integer(Npatterns)
  # 'H' denotes asin(sqrt(G))
  KtoL <- expression(sqrt(./pi))
  GtoH <- expression(asin(sqrt(.)))
  #
  rK <- seq(0, rmaxK, length=512)
  rG <- seq(0, rmaxG, length=512)
  areaW <- area.owin(W)
  #
  for(i in 1:Npatterns) {
    X <- rpoispp(lambda, win=W)
    nX <- npoints(X)
    lamX <- nX/areaW
    # generate common simulations
    XsimP <- XsimC <- vector(mode="list", length=nmc)
    for(j in 1:nmc) {
      XsimP[[j]] <- rpoispp(lamX, win=W)
      XsimC[[j]] <- runifpoint(nX, win=W)
    }
    # .......... Poisson simulations .....................
    # compute K functions and apply DCLF test
    dcKpois <- dclf.test(X, fun=Kest, r=rK, simulate=XsimP,
                         savefuns=TRUE,
                         nsim=nmc, verbose=FALSE)
    mKpois[i] <- as.integer(dcKpois$statistic$rank)
    # transform to L, apply DCLF test
    dcLpois <- dclf.test(dcKpois, transform=KtoL,
                         nsim=nmc, verbose=FALSE)
    mLpois[i] <- as.integer(dcLpois$statistic$rank)
    # compute G functions and apply DCLF test
    dcGpois <- dclf.test(X, fun=Gest, r=rG, simulate=XsimP,
                         savefuns=TRUE,
                         nsim=nmc, verbose=FALSE)
    mGpois[i] <- as.integer(dcGpois$statistic$rank)
    # stabilise variance, apply DCLF test
    dcHpois <- dclf.test(dcGpois, transform=GtoH,
                         nsim=nmc, verbose=FALSE)
    mHpois[i] <- as.integer(dcHpois$statistic$rank)

    # .......... conditional simulations .....................
    # compute K functions and apply DCLF test
    dcKcond <- dclf.test(X, fun=Kest, r=rK, simulate=XsimC,
                         savefuns=TRUE,
                         nsim=nmc, verbose=FALSE)
    mKcond[i] <- as.integer(dcKcond$statistic$rank)
    # transform to L, apply DCLF test
    dcLcond <- dclf.test(dcKcond, transform=KtoL,
                         nsim=nmc, verbose=FALSE)
    mLcond[i] <- as.integer(dcLcond$statistic$rank)
    # compute G functions and apply DCLF test
    dcGcond <- dclf.test(X, fun=Gest, r=rG, simulate=XsimC,
                         savefuns=TRUE,
                         nsim=nmc, verbose=FALSE)
    mGcond[i] <- as.integer(dcGcond$statistic$rank)
    # stabilise variance, apply DCLF test
    dcHcond <- dclf.test(dcGcond, transform=GtoH,
                         nsim=nmc, verbose=FALSE)
    mHcond[i] <- as.integer(dcHcond$statistic$rank)
    #
    if(verbose)
      progressreport(i, Npatterns)
  }
  return(as.matrix(data.frame(mKpois, mKcond,
                              mLpois, mLcond,
                              mGpois, mGcond,
                              mHpois, mHcond
                              )))
}


###################################################
### code chunk number 40: baddeley.Rnw:509-513
###################################################
if(recompute || !file.exists("CSRranks.rda")) {
  CSRranks <- CSR.ranks.dclf(1000, verbose=FALSE)
  save(CSRranks, file="CSRranks.rda")
} else load("CSRranks.rda")


###################################################
### code chunk number 41: baddeley.Rnw:519-524
###################################################
tab1 <- colMeans(CSRranks == 1)
tab1
tab1 <- matrix(tab1, ncol=2, byrow=TRUE)
dimnames(tab1) <- list(c("K", "L", "G", "H"), c("Uncond", "Cond"))
tab1


###################################################
### code chunk number 42: baddeley.Rnw:536-589
###################################################
MatClust.ranks.dclf <- function(Npatterns=1000, 
                                ...,
                                nmc=19,
                                kappa=0.005, r=14, mu=5, W=square(100),
                                rmaxK=25,
                                rmaxG=10,
                                verbose=TRUE) {
  mK <- mG <- mL <- mH <- rep(NA_integer_, Npatterns)
  # 'H' denotes asin(sqrt(G))
  KtoL <- expression(sqrt(./pi))
  GtoH <- expression(asin(sqrt(.)))
  #
  rK <- seq(0, rmaxK, length=512)
  rG <- seq(0, rmaxG, length=512)
  #
  for(i in 1:Npatterns) {
    # generate realisation of Matern Cluster process
    X <- rMatClust(kappa=kappa, r=r, mu=mu, win=W)
    # fit Matern Cluster model to X by minimum contrast
    fit <- kppm(X, ~1, clusters="MatClust")
    # generate simulations from fitted 'null' model for use in tests
    Xsim <- simulate(fit, nsim=nmc, verbose=FALSE, retry=0)
    # check there are no NULL outcomes i.e. simulation failures
    if(!any(unlist(lapply(Xsim, is.null)))) {
      # compute K functions and apply DCLF test
      dcK <- try(dclf.test(X, fun=Kest, r=rK, simulate=Xsim, 
                       savefuns=TRUE,
                       nsim=nmc, verbose=FALSE))
      if(!inherits(dcK, "try-error")) {
        mK[i] <- as.integer(dcK$statistic$rank)
        # transform to L, apply DCLF test
        dcL <- dclf.test(dcK, transform=KtoL,
                         nsim=nmc, verbose=FALSE)
        mL[i] <- as.integer(dcL$statistic$rank)
      }
      # compute G functions and apply DCLF test
      dcG <- try(dclf.test(X, fun=Gest, r=rG, simulate=Xsim,
                       savefuns=TRUE,
                       nsim=nmc, verbose=FALSE))
      if(!inherits(dcG, "try-error")) {
        mG[i] <- as.integer(dcG$statistic$rank)
        # stabilise variance, apply DCLF test
        dcH <- dclf.test(dcG, transform=GtoH,
                         nsim=nmc, verbose=FALSE)
        mH[i] <- as.integer(dcH$statistic$rank)
      }
    }
    #
    if(verbose)
      progressreport(i, Npatterns)
  }
  return(as.matrix(data.frame(mK, mL, mG, mH)))
}


###################################################
### code chunk number 43: baddeley.Rnw:597-601
###################################################
if(recompute || !file.exists("MatClustranks.rda")) {
  MatClustranks <- MatClust.ranks.dclf(5000, verbose=FALSE)
  save(MatClustranks, file="MatClustranks.rda")
} else load("MatClustranks.rda")


###################################################
### code chunk number 44: baddeley.Rnw:607-608
###################################################
colMeans(MatClustranks == 1, na.rm=TRUE)


